@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
 from django.core.urlresolvers import reverse
+from django.db.models.fields import FieldDoesNotExist
+from django.template import RequestContext, Context, Template
 from django.utils.encoding import force_unicode, StrAndUnicode
 from django.utils.datastructures import SortedDict
 from django.utils.text import capfirst
 from django.utils.html import escape
-from django.utils.safestring import mark_safe
-from django.template import RequestContext, Context, Template
-from django.db.models.fields import FieldDoesNotExist
+from django.utils.safestring import mark_safe, SafeData
 from itertools import ifilter, islice
+import warnings
 from .templatetags.django_tables2 import title
 from .utils import A, AttributeDict, Attrs, OrderBy, Sequence
 
@@ -77,6 +78,8 @@ class Column(object):
         self.visible = visible
         attrs = attrs or {}
         if not isinstance(attrs, Attrs):
+            warnings.warn('attrs must be Attrs object, not %s'
+                          % attrs.__class__.__name__, DeprecationWarning)
             attrs = Attrs(attrs)
         self.attrs = attrs
 
@@ -98,7 +101,10 @@ class Column(object):
         """
         The value used for the column heading (e.g. inside the ``<th>`` tag).
 
-        By default this equivalent to the column's :attr:`verbose_name`.
+        By default this titlises the column's :attr:`verbose_name`. If
+        ``verbose_name`` is an instance of ``SafeData``, it's used unmodified.
+
+        :returns: ``unicode`` or ``None``
 
         .. note::
 
@@ -109,7 +115,12 @@ class Column(object):
             object hence accessing that first) when this property doesn't
             return something useful.
         """
-        return title(self.verbose_name) if self.verbose_name else None
+        if self.verbose_name:
+            if isinstance(self.verbose_name, SafeData):
+                # If the author has used mark_safe, we're going to assume the
+                # author wants the value used verbatim.
+                return self.verbose_name
+            return title(self.verbose_name)
 
     def render(self, value):
         """
@@ -159,6 +170,8 @@ class CheckBoxColumn(Column):
         # should assign attributes to the `<input>` tag.
         attrs = attrs or Attrs()
         if not isinstance(attrs, Attrs):
+            warnings.warn('attrs must be Attrs object, not %s'
+                          % attrs.__class__.__name__, DeprecationWarning)
             attrs = Attrs(td__input=attrs)
         # This is done for backwards compatible too, there used to be a
         # ``header_attrs`` argument, but this has been deprecated. We'll
@@ -245,6 +258,8 @@ class LinkColumn(Column):
         # backwards compatible translation for naive attrs value
         attrs = attrs or Attrs()
         if not isinstance(attrs, Attrs):
+            warnings.warn('attrs must be Attrs object, not %s'
+                          % attrs.__class__.__name__, DeprecationWarning)
             attrs = Attrs(a=attrs)
         extra['attrs'] = attrs
         super(LinkColumn, self).__init__(**extra)
@@ -291,7 +306,7 @@ class LinkColumn(Column):
                                      if isinstance(self.current_app, A)
                                      else self.current_app)
         html = u'<a href="{url}" {attrs}>{value}</a>'.format(
-            url=reverse(**params),
+            url=escape(reverse(**params)),
             attrs=AttributeDict(self.attrs.get('a', {})).as_html(),
             value=escape(value)
         )
@@ -331,12 +346,15 @@ class TemplateColumn(Column):
         self.template_code = template_code
 
     def render(self, record, table, **kwargs):
-        t = Template(self.template_code)
-        if hasattr(table, 'request'):
-            context = RequestContext(table.request, {'record': record})
-        else:
-            context = Context({'record': record})
-        return t.render(context)
+        # If the table is being rendered using `render_table`, it hackily
+        # attaches the context to the table as a gift to `TemplateColumn`. If
+        # the table is being rendered via `Table.as_html`, this won't exist.
+        context = getattr(table, 'context', Context())
+        context.update({'record': record})
+        try:
+            return Template(self.template_code).render(context)
+        finally:
+            context.pop()
 
 
 class BoundColumn(object):
@@ -431,7 +449,18 @@ class BoundColumn(object):
         """
         The value that should be used in the header cell for this column.
         """
-        return self.column.header or title(self.verbose_name)
+        # favour Column.header
+        column_header = self.column.header
+        if column_header:
+            return column_header
+        # fall back to automatic best guess
+        verbose_name = self.verbose_name  # avoid calculating multiple times
+        if isinstance(verbose_name, SafeData):
+            # If the verbose_name has come from a model field, it's possible
+            # that the author used mark_safe to include HTML in the value. If
+            # this is the case, we leave it verbatim.
+            return verbose_name
+        return title(verbose_name)
 
     @property
     def name(self):
@@ -479,6 +508,9 @@ class BoundColumn(object):
         then get the last field in the accessor (i.e. stop when the
         relationship turns from ORM relationships to object attributes [e.g.
         person.upper should stop at person]).
+
+        If the model field's ``verbose_name`` is a ``SafeData``, it's used
+        unmodified.
         """
         # Favor an explicit defined verbose_name
         if self.column.verbose_name:
@@ -653,7 +685,8 @@ class BoundColumns(object):
             for column in self.iterall():
                 if column.name == index:
                     return column
-            raise KeyError("Column with name '%s' does not exist." % index)
+            raise KeyError("Column with name '%s' does not exist; "
+                           "choices are: %s" % (index, self.names()))
         else:
             raise TypeError(u'row indices must be integers or str, not %s'
                             % index.__class__.__name__)
