@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 import copy
+from django.conf import settings
 from django.core.paginator import Paginator
 from django.http import Http404
 from django.utils.datastructures import SortedDict
@@ -371,7 +372,10 @@ class Table(StrAndUnicode):
             try:
                 return self.paginator.page(self._page_number)
             except:
-                raise Http404(sys.exc_info()[1])
+                if settings.DEBUG:
+                    raise
+                else:
+                    raise Http404(sys.exc_info()[1])
 
     @property
     def per_page_field(self):
