@@ -10,6 +10,7 @@ from django.template import Context, Template
 from django.core.exceptions import ImproperlyConfigured
 from django.utils.translation import ugettext_lazy
 from django.utils.translation import ugettext
+from django.utils.safestring import mark_safe, SafeData
 from itertools import chain
 from xml.etree import ElementTree as ET
 from .app.models import Person
@@ -64,6 +65,24 @@ def column_header_should_use_titlised_verbose_name():
     assert table.columns["acronym"].header == "Has FBI Help"
 
 
+@general.test
+def should_support_safe_verbose_name():
+    class SimpleTable(tables.Table):
+        safe = tables.Column(verbose_name=mark_safe("<b>Safe</b>"))
+
+    table = SimpleTable([])
+    assert isinstance(table.columns["safe"].header, SafeData)
+
+
+@general.test
+def should_support_safe_verbose_name_via_model():
+    class PersonTable(tables.Table):
+        safe = tables.Column()
+
+    table = PersonTable(Person.objects.all())
+    assert isinstance(table.columns["safe"].header, SafeData)
+
+
 @general.test
 def sortable():
     class SimpleTable(tables.Table):
@@ -277,10 +296,10 @@ def kwargs():
 @linkcolumn.test
 def html_escape_value():
     class PersonTable(tables.Table):
-        name = tables.LinkColumn("occupation", kwargs={"pk": A("pk")})
+        name = tables.LinkColumn("escaping", kwargs={"pk": A("pk")})
 
-    html = PersonTable([{"name": "<brad>", "pk": 1}]).as_html()
-    assert "<brad>" not in html
+    table = PersonTable([{"name": "<brad>", "pk": 1}])
+    assert table.rows[0]["name"] == '<a href="/&amp;&#39;%22/1/" >&lt;brad&gt;</a>'
 
 
 @linkcolumn.test
@@ -305,4 +324,18 @@ def a_attrs_should_be_supported():
                                            "title": "Occupation Title"}
 
 
-columns = Tests([checkboxcolumn, general, linkcolumn])
+templatecolumn = Tests()
+
+
+@templatecolumn.test
+def should_handle_context_on_table():
+    class TestTable(tables.Table):
+        col = tables.TemplateColumn("{{ record.col }}{{ STATIC_URL }}")
+
+    table = TestTable([{"col": "brad"}])
+    assert table.rows[0]["col"] == "brad"
+    table.context = Context({"STATIC_URL": "/static/"})
+    Assert(table.rows[0]["col"]) == "brad/static/"
+
+
+columns = Tests([checkboxcolumn, general, linkcolumn, templatecolumn])
